/*
     ------------------------------------------------
     LIA_SCT: A DECISION-TREE BASED STRING CLASSIFIER
     ------------------------------------------------
 
     Copyright (C) 2001 FREDERIC BECHET
 
     ..................................................................
 
     This file is part of LIA_SCT
 
     LIA_SCT is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
     ..................................................................
 
     For any publication related to scientific work using LIA_SCT,
     the following reference paper must be mentioned in the bibliography: 
         
     Bechet F., Nasr A., Genet F. 2000, "Tagging unknown proper names
     using decision trees", in 38th Annual Meeting of the Association for
     Computational Linguistics (ACL), Hong-Kong, China, 2000, pp. 77-84
     ..................................................................
                               
     Contact :
               FREDERIC BECHET - LIA - UNIVERSITE D'AVIGNON
               AGROPARC BP1228 84911  AVIGNON  CEDEX 09  FRANCE
               frederic.bechet@lia.univ-avignon.fr
     ..................................................................
*/
/*  Test de mixture des LM noeuds selon la formule suivante :
    P(phrase)=Produit(P(bigram))
    P(bigram)=P(bi|bi-1)=Somme_j(P_LMj(bi|bi-1)*P(LMj|bi bi-1))
    et P(LMj|bi bi-1)=|LMj(bi bi-1)|/|Somme_k(LMk(bi bi-1))|
    On a besoin en entree :
      - un descripteur (format lia_sctgram)
      - une liste de numero de feuille (un numero par ligne)  */
/*  FRED 0600  Modif 1200  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <math.h>
#include <libgram.h>
#include <lia_tree_avl.h>

#define TailleLigne	800

#define MIN(a,b)	((a)>(b)?(b):(a))
#define MAX(a,b)	((a)>(b)?(a):(b))

int debug=2;

int verbose=0;

void ERREUR(char *ch1,char *ch2)
{
fprintf(stderr,"ERREUR : %s %s\n",ch1,ch2);
exit(0);
}

/* exemple de fichier descripteur :

# francaise phonetisee au LIA et decrite sur 2 niveaux :
# phoneme et cohorte
#
# FRED 0600

NB_NIVEAUX      2
INDICE_ML       1
INDICE_KL       3
CORPUS_TRAIN    ./DATA/list_localite.train.fmt
CORPUS_TEST     ./DATA/list_localite.test.fmt
CORPUS_DEV      NULL
ARBRE           ./DATA/list_localite.arbre
FEUILLE_REP     ./FEUILLE
TEST_REP        ./TEST
NOEUD_REP       ./NOEUD
SCRIPT_REP      ./SCRIPT
TAILLE_FEUILLE  2000
*/

/* variables globales pour la description des donnees */

/* on ne limite pas la taille de l'arbre */
#define PROF_MAXI	10000

int NB_NIVEAUX,INDICE_KL,INDICE_ML,TAILLE_FEUILLE;
char CORPUS_TRAIN[TailleLigne],CORPUS_TEST[TailleLigne],CORPUS_DEV[TailleLigne],
     ARBRE[TailleLigne],FEUILLE_REP[TailleLigne],TEST_REP[TailleLigne],
     NOEUD_REP[TailleLigne],SCRIPT_REP[TailleLigne];

/*................................................................*/

/* gestion de la divergence de Kullback */

typedef struct
	{
	float pb;
	wrd_index_t code;
	} type_kl;

int NbComposanteKL; /* taille des vecteur de KL */

/* un vecteur kl par feuille */

type_kl **TablKL;

/*................................................................*/

/* stockage des numeros de feuilles */
int NbFeuilles;
int *TablFeuille;

/* 3 arbre avl (bigramme+mot_train+mot_test) par feuille + 1 pour la reference=Somme des corpus */
lia_avl_t *TablBigramAVL,*TablGrafTrainAVL,*TablGrafTestAVL,RefAVL=NULL;

/* un modele ARPA par feuille + 1 pour la reference */
int *TablML,RefML;

/* on stocke 2 coeff par feuille : le melange modele ref + modele noeud
   obtenu avec la commande 'interpolate' */
typedef struct
	{
	double coeff_ref,coeff_noeud;
	} type_coeff;

type_coeff *TablCoeff;

/* on stocke le nb max de bigram */
int NbTotalBigram;

/* on a le lexique des modeles ngram */
ty_lexique LexiqueGram;

void lecture_liste_feuille(char *nomfic)
{
FILE *file,*filecoeff;
char ch[TailleLigne];

if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
for(NbFeuilles=0;fgets(ch,TailleLigne,file);NbFeuilles++);
fseek(file,0,SEEK_SET);
TablFeuille=(int*)malloc(sizeof(int)*NbFeuilles);
TablBigramAVL=(lia_avl_t *)malloc(sizeof(lia_avl_t)*NbFeuilles);
TablGrafTrainAVL=(lia_avl_t *)malloc(sizeof(lia_avl_t)*NbFeuilles);
TablGrafTestAVL=(lia_avl_t *)malloc(sizeof(lia_avl_t)*NbFeuilles);
TablML=(int*)malloc(sizeof(int)*NbFeuilles);
TablCoeff=(type_coeff *)malloc(sizeof(type_coeff)*NbFeuilles);
for(NbFeuilles=0;fgets(ch,TailleLigne,file);NbFeuilles++)
 {
 TablBigramAVL[NbFeuilles]=NULL;
 TablGrafTrainAVL[NbFeuilles]=NULL;
 TablGrafTestAVL[NbFeuilles]=NULL;
 if (sscanf(ch,"%d",&(TablFeuille[NbFeuilles]))!=1) ERREUR("Mauvais format dans le fichier :",nomfic);
 /* on lit les coeffs */
 sprintf(ch,"%s/noeud_n%d.coeff",NOEUD_REP,TablFeuille[NbFeuilles]);
 if (!(filecoeff=fopen(ch,"rt"))) ERREUR("pas de fichier coeff :",ch);
 fgets(ch,TailleLigne,filecoeff);
 if (sscanf(ch,"%lf %lf",&(TablCoeff[NbFeuilles].coeff_ref),&(TablCoeff[NbFeuilles].coeff_noeud))!=2)
  ERREUR("mauvais format dans le fichier coeff","");
 fclose(filecoeff);
 }
 fclose(file);
}

/*................................................................*/

/* compile les fichiers arpa des feuilles et du corpus general (0) */

void compile_arpa_noeud()
{
char ch[TailleLigne];
int i;

fprintf(stderr,"Compilation des modeles ARPA :\n");

/* on compile le fichier general indice=0 */
fprintf(stderr,"\t- modele n0\n");
sprintf(ch,"$LIA_TOOLTALN_REP/bin/compile_arpa -lexique %s.niv%d %s/noeud_n0.arpa log_10 2g -dicho",
 ARBRE,INDICE_ML,NOEUD_REP);
system(ch);

for(i=0;i<NbFeuilles;i++)
 {
 fprintf(stderr,"\t- modele n%d\n",TablFeuille[i]);
 sprintf(ch,"$LIA_TOOLTALN_REP/bin/compile_arpa -lexique %s.niv%d %s/noeud_n%d.arpa log_10 2g -dicho",
  ARBRE,INDICE_ML,NOEUD_REP,TablFeuille[i]);
 system(ch);
 }

fprintf(stderr,"Termine\n");

fprintf(stderr,"Calcul de l'interpolation : modele general / modele noeuds\n");
sprintf(ch,"$LIA_SCT_REP/script/lance_fprobs %s %s",ARBRE,NOEUD_REP);
system(ch);
fprintf(stderr,"Termine\n");
}

/*................................................................*/

/* charge les modeles arpa deja compiles */

void charge_modele_arpa()
{
char ch[TailleLigne];
int i;
err_t retour;

fprintf(stderr,"Lecture des ML -> ");
sprintf(ch,"%s/noeud_n0.arpa",NOEUD_REP);
if (gram_module_init(ch,&RefML,2,0)!=CORRECT) ERREUR("Mauvaise lecture du modele :",ch);
for(i=0;i<NbFeuilles;i++)
 {
 sprintf(ch,"%s/noeud_n%d.arpa",NOEUD_REP,TablFeuille[i]);
 if ((retour=gram_module_init(ch,&(TablML[i]),2,0))!=CORRECT)
  {
  fprintf(stderr,"retour=%d\n",retour);
  ERREUR("Mauvaise lecture du modele :",ch);
  }
 }
fprintf(stderr,"Termine\n");
}

/*................................................................*/

/* remplissage des arbres AVL avec les bigrammes de chaque fichier */

void rempli_arbre()
{
int nb,i,feuille;
char ch[TailleLigne*10],*pt,mot0[TailleLigne],mot1[TailleLigne],bigram[TailleLigne],nomfic[TailleLigne];
FILE *file;

for(feuille=0;feuille<NbFeuilles;feuille++)
 {
 /* on traite les corpus de train */
 sprintf(nomfic,"%s/noeud_n%d.corpus",NOEUD_REP,TablFeuille[feuille]);
 if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
 for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
  {
  mot0[0]=mot1[0]='\0';
  pt=strtok(ch," \t\n");
  TablGrafTrainAVL[feuille]=lia_ajoute_element_avl(TablGrafTrainAVL[feuille],pt,True,NULL);
  pt=strtok(NULL," \t\n");
  if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
  while (pt)
   {
   for(i=0;i<NB_NIVEAUX;i++)
    {
    if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
    if ((i+1)==INDICE_ML)
     {
     strcpy(mot0,mot1); strcpy(mot1,pt);
     if (mot0[0]!='\0')
      {
      sprintf(bigram,"%s|%s",mot0,mot1);
      TablBigramAVL[feuille]=lia_ajoute_element_avl(TablBigramAVL[feuille],bigram,True,NULL);
      RefAVL=lia_ajoute_element_avl(RefAVL,bigram,True,NULL);

      NbTotalBigram++;
      }
     }
    pt=strtok(NULL," \t\n");
    }
   }
  }
 fclose(file);

 /* on traite les corpus de test */
 sprintf(nomfic,"%s/noeud_n%d.test",NOEUD_REP,TablFeuille[feuille]);
 if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
 for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
  {
  pt=strtok(ch," \t\n");
  if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
  TablGrafTestAVL[feuille]=lia_ajoute_element_avl(TablGrafTestAVL[feuille],pt,True,NULL);
  }
 fclose(file);
 }
}

/*................................................................*/

/* mixture des modeles avec la proba P(modele|bigram) */

void trouve_coeff(double *tablalpha,int nbgraf,char **tablgraf)
{
int i,j,k,nb_i,nb_ref,nbreel=0;
wrd_index_t m0,m1;
char bigram[TailleLigne];

for(i=0;i<NbFeuilles;i++) tablalpha[i]=(double)0;

for(i=0;i<nbgraf;i++)
 {
 if (!Mot2Code(tablgraf[i],&m0,LexiqueGram)) ERREUR("Mot inconnu :",tablgraf[i]);
 for(j=0;j<nbgraf;j++)
  {
  sprintf(bigram,"%s|%s",tablgraf[i],tablgraf[j]);
  if (lia_recherche_avl(RefAVL,bigram,&nb_ref))
   {
   nbreel++;

   if (!Mot2Code(tablgraf[j],&m1,LexiqueGram)) ERREUR("Mot inconnu :",tablgraf[j]);
   for(k=0;k<NbFeuilles;k++)
    {
    if (!lia_recherche_avl(TablBigramAVL[k],bigram,&nb_i)) nb_i=0;
    tablalpha[k]+=(double)nb_i/(double)nb_ref;
    }
   }
  }
 }
for(i=0;i<NbFeuilles;i++) tablalpha[i]/=(double)nbreel;

}

double mixture(char *ch,wrd_index_t m0,wrd_index_t m1,char *mot0,char *mot1)
{
int i,nb_ref,nb_i,j;
double pb1,pb2,total;
err_t retour;

total=(double)0;

for (i=0;i<NbFeuilles;i++)
 {
 /* P_LMi(m1|m0) */
 retour=gram_proba_to_bigram(&pb1,m0,m1,TablML[i],0);
 if (retour!=CORRECT)
  { fprintf(stderr,"ERREUR : retour=%d pour m0=%d m1=%d\n",retour,m0,m1); exit(0); }
 pb1=exp(pb1*log(10.0));

 /* P(modele|m0) */
 if (!lia_recherche_avl(RefAVL,mot0,&nb_ref)) nb_ref=0;
 if (!lia_recherche_avl(TablBigramAVL[i],mot0,&nb_i)) nb_i=0;
 if (nb_ref) pb2=(double)nb_i/(double)nb_ref; else pb2=0;

 total+=(pb1*pb2);
 }
return log10(total); 
}

double mixture2(char *ch,wrd_index_t m0,wrd_index_t m1,double *tablalpha)
{
int i,nb_ref,nb_i,j;
double pb1,pb2,total;
err_t retour;

total=(double)0;

for (i=0;i<NbFeuilles;i++)
 {
 /* P_LMi(m1|m0) */
 retour=gram_proba_to_bigram(&pb1,m0,m1,TablML[i],0);
 if (retour!=CORRECT)
  { fprintf(stderr,"ERREUR : retour=%d pour m0=%d m1=%d\n",retour,m0,m1); exit(0); }
 pb1=exp(pb1*log(10.0));

 total+=pb1*tablalpha[i];
 }

return log10(total);
}

/*................................................................*/

/* analyse des fichiers tests et calcule de la perplexite */

void analyse_perplexite()
{
int nb,i,feuille,nb_big,nb_ref,nb_mis,nbgraf,nb_big_local;
char ch[TailleLigne*10],*pt,mot0[TailleLigne],mot1[TailleLigne],bigram[TailleLigne],nomfic[TailleLigne];
FILE *file;
double total_ref,total_opt,total_mix,pb,pb_ref,pp_ref,pp_opt,pp_mix,*tablalpha,
       local_ref,local_opt,local_mix;
wrd_index_t m0,m1;
err_t retour;
char **tablgraf;

printf("\n\n"); system("banner RESULTAT"); printf("\n\n");

sprintf(ch,"%s.niv%d",ARBRE,INDICE_ML);
if (!(file=fopen(ch,"rt"))) ERREUR("can't open :",ch);
for(nbgraf=0;fgets(ch,TailleLigne,file);nbgraf++);
tablgraf=(char**)malloc(sizeof(char*)*nbgraf);
fseek(file,0,SEEK_SET);
for(nbgraf=0;fgets(ch,TailleLigne,file);nbgraf++)
 {
 /* la graf est dans le 2em champs */
 pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
 tablgraf[nbgraf]=(char*)strdup(pt);
 }
fclose(file);

/* cherche les coeff alpha */
tablalpha=(double*)malloc(sizeof(double)*NbFeuilles);
trouve_coeff(tablalpha,nbgraf,tablgraf);

total_ref=total_opt=total_mix=(double)0;

nb_big=nb_mis=0;

for(feuille=0;feuille<NbFeuilles;feuille++)
 {
 local_ref=local_opt=local_mix=(double)0;
 nb_big_local=0;
 sprintf(nomfic,"%s/noeud_n%d.test",NOEUD_REP,TablFeuille[feuille]);
 if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);

 for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
  {
  mot0[0]=mot1[0]='\0';
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
  while (pt)
   {
   for(i=0;i<NB_NIVEAUX;i++)
    {
    if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
    if ((i+1)==INDICE_ML)
     {
     strcpy(mot0,mot1); strcpy(mot1,pt);
     if (mot0[0]!='\0')
      {
      /* on est sur un bigramme !! */
      sprintf(bigram,"%s|%s",mot0,mot1);

      /* on ne garde que les bigram existant dans le train pour le calcul de pp */
      if ((0)||(lia_recherche_avl(RefAVL,bigram,&nb_ref)))
       {
       nb_big_local++;

       if (!Mot2Code(mot0,&m0,LexiqueGram)) ERREUR("Mot inconnu :",mot0);
       if (!Mot2Code(mot1,&m1,LexiqueGram)) ERREUR("Mot inconnu :",mot1);

       retour=gram_proba_to_bigram(&pb_ref,m0,m1,RefML,0);
       if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (RefML) GRAM_CODE_UNKNOWN sur :",bigram);
       if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }
       local_ref+=(-pb_ref);

       retour=gram_proba_to_bigram(&pb,m0,m1,TablML[feuille],0);
       if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (ML feuille) GRAM_CODE_UNKNOWN sur :",bigram);
       if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }
       local_opt+=(-pb);
       local_mix+=(-(TablCoeff[feuille].coeff_ref*pb_ref+TablCoeff[feuille].coeff_noeud*pb));

       /*local_mix+=(-mixture2(bigram,m0,m1,tablalpha));*/
       /*local_mix+=(-mixture(bigram,m0,m1,mot0,mot1));*/
       }
      else nb_mis++;
      }
     }
    pt=strtok(NULL," \t\n");
    }
   }
  }
  
 pp_ref=exp(local_ref/(double)nb_big_local*log(10.0));
 pp_opt=exp(local_opt/(double)nb_big_local*log(10.0));
 pp_mix=exp(local_mix/(double)nb_big_local*log(10.0));
 printf("Corpus[%d] : PP_ref=%.2lf \t PP_opt=%.2lf \t (gain=%.2lf%%)     \t PP_mix=%.2lf \t (gain=%.2lf%%)\n",
  TablFeuille[feuille],pp_ref,pp_opt,(pp_ref-pp_opt)*(double)100/pp_ref,pp_mix,(pp_ref-pp_mix)*(double)100/pp_ref);

 nb_big+=nb_big_local;
 total_ref+=local_ref; total_opt+=local_opt; total_mix+=local_mix;
 fclose(file);
 }
pp_ref=exp(total_ref/(double)nb_big*log(10.0));
pp_opt=exp(total_opt/(double)nb_big*log(10.0));
pp_mix=exp(total_mix/(double)nb_big*log(10.0));

printf("Nb de bigram utilises : %d (sur un total de %d avec %d absent)\n",nb_big,nb_big+nb_mis,nb_mis);
printf("Perplexite avec le modele reference : %.2lf\n",pp_ref);
printf("Perplexite avec le modele optimal   : %.2lf\n",pp_opt);
printf("Perplexite avec le modele mixture   : %.2lf\n",pp_mix);
printf("\tgain opt/ref = %.2lf%%\n",(pp_ref-pp_opt)*(double)100/pp_ref);
printf("\tgain mix/ref = %.2lf%%\n",(pp_ref-pp_mix)*(double)100/pp_ref);
}

/*................................................................*/

/* test de coherence */

void coherence()
{
char ch[TailleLigne],**tablgraf,*pt,bigram[TailleLigne];
FILE *file;
int nbgraf,i,j,nb_ref,k;
double pb,total_ref,total_mix,*tabl_total,*tablalpha;
wrd_index_t m0,m1;
err_t retour;

tabl_total=(double*)malloc(sizeof(double)*NbFeuilles);
sprintf(ch,"%s.niv%d",ARBRE,INDICE_ML);
if (!(file=fopen(ch,"rt"))) ERREUR("can't open :",ch);
for(nbgraf=0;fgets(ch,TailleLigne,file);nbgraf++);
tablgraf=(char**)malloc(sizeof(char*)*nbgraf);
fseek(file,0,SEEK_SET);
for(nbgraf=0;fgets(ch,TailleLigne,file);nbgraf++)
 {
 /* la graf est dans le 2em champs */
 pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
 tablgraf[nbgraf]=(char*)strdup(pt);
 }
fclose(file);

/* cherche les coeff alpha */
tablalpha=(double*)malloc(sizeof(double)*NbFeuilles);
trouve_coeff(tablalpha,nbgraf,tablgraf);
total_mix=0;
for(i=0;i<NbFeuilles;i++)
 {
 total_mix+=tablalpha[i];
 printf("coeff alpha %d = %.4lf\n",i,tablalpha[i]);
 }
printf("Total alpha = %.2lf\n",total_mix);

/* test sur le modele general */
for(i=0;i<nbgraf;i++)
 {
 if (!Mot2Code(tablgraf[i],&m0,LexiqueGram)) ERREUR("Mot inconnu :",tablgraf[i]);
 total_ref=total_mix=(double)0;
 for(k=0;k<NbFeuilles;k++) tabl_total[k]=(double)0;
 for(j=0;j<nbgraf;j++)
  {
  sprintf(bigram,"%s|%s",tablgraf[i],tablgraf[j]);
  if ((1)||(lia_recherche_avl(RefAVL,bigram,&nb_ref)))
   {
   if (!Mot2Code(tablgraf[j],&m1,LexiqueGram)) ERREUR("Mot inconnu :",tablgraf[j]);
   retour=gram_proba_to_bigram(&pb,m0,m1,RefML,0);
   if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (RefML) GRAM_CODE_UNKNOWN sur :",bigram);
   if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }
   total_ref+=exp(pb*log(10.0));
   /*total_mix+=exp(mixture(bigram,m0,m1,tablgraf[i],tablgraf[j])*log(10.0));*/
   total_mix+=exp(mixture2(bigram,m0,m1,tablalpha)*log(10.0));
   for(k=0;k<NbFeuilles;k++)
    {
    retour=gram_proba_to_bigram(&pb,m0,m1,TablML[k],0);
    if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (ML Feuille) GRAM_CODE_UNKNOWN sur :",bigram);
    if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }
    tabl_total[k]+=exp(pb*log(10.0));
    }
   }
  }
 printf("Pour le mot %s : total_ref=%.2lf  \t  total_mix=%.2lf\n",
	tablgraf[i],total_ref,total_mix);
 /*
 for(k=0;k<NbFeuilles;k++) printf("ML(%d)=%.2lf ",TablFeuille[k],tabl_total[k]);
 printf("\n");
 */
 }
}

/*................................................................*/

/* genere les fichiers .kl sur les feuilles */

float calcule_kl(type_kl *v1,type_kl *v2)
{
int i;
double d;
for(d=(double)0,i=0;i<NbComposanteKL;i++)
 {
 /*
 if ((v1[i].pb!=(float)0)&&(v2[i].pb!=(float)0))
  fprintf(stderr,"XX : pi=%lf , qi=%lf , pi/qi=%lf , log(pi/qi)=%lf\n",
   (double)v1[i].pb,(double)v2[i].pb,((double)v1[i].pb)/((double)v2[i].pb),log(((double)v1[i].pb)/((double)v2[i].pb)));
 */

 if ((v1[i].pb!=(float)0)&&(v2[i].pb!=(float)0)) d+=((double)v1[i].pb)*log(((double)v1[i].pb)/((double)v2[i].pb));
 }
return (float)d;
}

int acces_composante_kl(type_kl *vecteur_kl,wrd_index_t code)
{
int deb,med,fin;
deb=0; fin=NbComposanteKL-1;
while (deb<fin)
 {
 med=(deb+fin)/2;
 if (vecteur_kl[med].code==code) return med;
 if (vecteur_kl[med].code<code) deb=med+1; else fin=med-1;
 }
if ((deb==fin)&&(vecteur_kl[deb].code==code)) return deb;
fprintf(stderr,"ERREUR : je ne trouve pas le code '%d' dans le vecteur kl !!\n",code);
exit(0); return 0;
}

int compar_kl(const void *a,const void *b)
{
type_kl *c,*d;
c=(type_kl *)a; d=(type_kl *)b;
if (c->code>d->code) return 1; else return -1;
}

void genere_kl()
{
int nb,i,j,feuille,nb_mot;
char ch[TailleLigne*10],*pt,nomfic[TailleLigne];
FILE *file;
wrd_index_t m0;
type_kl *vecteur_kl;
ty_lexique lex_kl;

sprintf(nomfic,"%s.niv%d.des",ARBRE,INDICE_KL);
if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
if (!fgets(ch,TailleLigne,file)) ERREUR("mauvais fichier :",nomfic);
pt=strtok(ch," \t\n");
if (sscanf(pt,"%d",&NbComposanteKL)!=1) ERREUR("mauvais fichier :",nomfic);
fclose(file);
vecteur_kl=(type_kl *)malloc(sizeof(type_kl)*NbComposanteKL);

/* on rempli les codes */
sprintf(nomfic,"%s.niv%d",ARBRE,INDICE_KL);
if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
for (nb=0;fgets(ch,TailleLigne,file);nb++)
 {
 if (nb>=NbComposanteKL) ERREUR("Pb dans le lexique : ",nomfic);
 pt=strtok(ch," \t\n");
 if (sscanf(pt,"%d",&i)!=1) ERREUR("mauvais fichier :",nomfic);
 vecteur_kl[nb].code=(wrd_index_t)i;
 vecteur_kl[nb].pb=0;
 }
fclose(file);

/* on tri les codes */
qsort(vecteur_kl,NbComposanteKL,sizeof(type_kl),compar_kl);
/*for(i=0;i<NbComposanteKL;i++) fprintf(stderr,"TEST_KL : %d\n",vecteur_kl[i].code);*/

sprintf(nomfic,"%s.niv%d",ARBRE,INDICE_KL);
lex_kl=ChargeLexique(nomfic);

for(feuille=0;feuille<NbFeuilles;feuille++)
 {
 nb_mot=0;
 sprintf(nomfic,"%s/noeud_n%d.corpus",NOEUD_REP,TablFeuille[feuille]);
 if (!(file=fopen(nomfic,"rt"))) ERREUR("can't open :",nomfic);
 for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
  while (pt)
   {
   for(i=0;i<NB_NIVEAUX;i++)
    {
    if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb,nomfic); exit(0); }
    if ((i+1)==INDICE_KL)
     {
     /* le mot est dans pt */
     if (!Mot2Code(pt,&m0,lex_kl))
      {
      fprintf(stderr,"Warning : '%s' inconnu dans 'lex_kl'\n",pt);
      }
     else
      {
      j=acces_composante_kl(vecteur_kl,m0);
      vecteur_kl[j].pb++;
      }
     }
    pt=strtok(NULL," \t\n");
    nb_mot++;
    }
   }
  }
 fclose(file);
 /* on calcule les probas */
 for(i=0;i<NbComposanteKL;i++) vecteur_kl[i].pb/=(float)nb_mot;
 /* on sauvegarde le vecteur */
 sprintf(nomfic,"%s/noeud_n%d.kl",NOEUD_REP,TablFeuille[feuille]);
 if (!(file=fopen(nomfic,"wb"))) ERREUR("can't write in :",nomfic);
 fwrite(vecteur_kl,sizeof(type_kl),NbComposanteKL,file);
 fclose(file);
 
 for(i=0;i<NbComposanteKL;i++)
  fprintf(stderr,"KLTEST : vecteur_kl[%d].code=%d pb=%f\n",i,vecteur_kl[i].code,vecteur_kl[i].pb);
 
 }
}

void charge_kl()
{
FILE *file;
char ch[TailleLigne],*pt;
int i,j;

fprintf(stderr,"chargement des fichiers kl -> ");

sprintf(ch,"%s.niv%d.des",ARBRE,INDICE_KL);
if (!(file=fopen(ch,"rt"))) ERREUR("can't open :",ch);
if (!fgets(ch,TailleLigne,file)) ERREUR("mauvais fichier descripteur lexique","");
pt=strtok(ch," \t\n");
if (sscanf(pt,"%d",&NbComposanteKL)!=1) ERREUR("mauvais fichier descripteur lexique","");
fclose(file);

TablKL=(type_kl**)malloc(sizeof(type_kl*)*NbFeuilles);
for(i=0;i<NbFeuilles;i++)
 {
 /* on lit les vecteurs kl */
 sprintf(ch,"%s/noeud_n%d.kl",NOEUD_REP,TablFeuille[i]);
 TablKL[i]=(type_kl*)malloc(sizeof(type_kl)*NbComposanteKL);
 if (!(file=fopen(ch,"rb"))) ERREUR("pas de fichier kl :",ch);
 fread(TablKL[i],sizeof(type_kl),NbComposanteKL,file);
 fclose(file);
 /*for(j=0;j<NbComposanteKL;j++) fprintf(stderr,"KLTEST : v[%d].code=%d\n",j,TablKL[i][j].code);*/
 }

fprintf(stderr,"termine\n");
}

/*................................................................*/

/* determine la feuille la + proche (en terme de perplexite) de
   chaque mot d'un fichier (au format .fmt en cours) et verifie
   si le mot appartient au corpus de la feuille  */

#define PROPORTION	100

typedef struct
	{
	int feuille;
	double pp;
	} type_pp;

int compar_perplex(const void *a,const void *b)
{
type_pp *c,*d;
c=(type_pp*)a; d=(type_pp*)b;
if (c->pp>d->pp) return 1;
if (c->pp<d->pp) return -1;
return 0;
}

void determine_corpus(char *fichtest)
{
int nb,i,j,k,feuille,nb_big,nb_good,nb_baad,nb_nspp,nb_mot,place=0,si_trouvable,nb_trouvable=0,sibon,*nbbigfeuille;
char ch[TailleLigne*10],*pt,mot0[TailleLigne],mot1[TailleLigne],bigram[TailleLigne],graf[TailleLigne],
     sauvreference[TailleLigne];
FILE *file;
wrd_index_t m0,m1;
err_t retour;
double ref_perplex,pb,pb_ref,pp_min;
type_pp *tabl_perplex;
float lp,fr;

int n_premiers; /* relatif de NbFeuille */

n_premiers=(PROPORTION*NbFeuilles)/100;

nbbigfeuille=(int*)malloc(sizeof(int)*NbFeuilles);
tabl_perplex=(type_pp*)malloc(sizeof(type_pp)*NbFeuilles);

nb_good=nb_baad=nb_nspp=0;

if (!(file=fopen(fichtest,"rt"))) ERREUR("can't open :",fichtest);
for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
 {
 nb_mot=0;
 si_trouvable=0;

 for(feuille=0;feuille<NbFeuilles;feuille++)
  {
  nbbigfeuille[feuille]=0;
  nbbigfeuille[feuille]=0;
  tabl_perplex[feuille].pp=(double)0;
  tabl_perplex[feuille].feuille=feuille;
  }
 ref_perplex=(double)0;
 nb_big=0;

 mot0[0]=mot1[0]='\0';
 pt=strtok(ch," \t\n");

 /* on recopie jusqu'a '_' : MODIF ISIP 16 01 01 */
 /*strcpy(graf,pt);*/
 strcpy(sauvreference,pt);
 for(k=0;(pt[k])&&(pt[k]!='_');k++) graf[k]=pt[k]; graf[k]='\0';

 pt=strtok(NULL," \t\n");
 if (pt==NULL) { fprintf(stderr,"ERREUR1 : ligne %d incorrect dans le fichier %s\n",nb,fichtest); exit(0); }

 /*if (!strcmp(graf,"art_249_18")) verbose=2; else verbose=0;*/

 while (pt)
  {
  for(i=0;i<NB_NIVEAUX;i++)
   {
   if (pt==NULL) { fprintf(stderr,"ERREUR2 : ligne %d incorrect dans le fichier %s\n",nb,fichtest); exit(0); }

   if ((i+1)==INDICE_ML)
    {
    strcpy(mot0,mot1); strcpy(mot1,pt);
    if (mot0[0]!='\0')
     {
     /* on est sur un bigramme !! */
     sprintf(bigram,"%s|%s",mot0,mot1);

     if (verbose>2) printf("bigramme : %s\n",bigram);

     if (!Mot2Code(mot0,&m0,LexiqueGram))
      {
      /*ERREUR("Mot inconnu :",mot0);*/
      fprintf(stderr,"Warning : %s inconnu\n",mot0);

      if (!Mot2Code("<UNK>",&m0,LexiqueGram)) ERREUR("<UNK> n'est pas dans le lexique","");
      }
     if (!Mot2Code(mot1,&m1,LexiqueGram))
      {
      /*ERREUR("Mot inconnu :",mot1);*/
      fprintf(stderr,"Warning : %s inconnu\n",mot1);

      if (!Mot2Code("<UNK>",&m1,LexiqueGram)) ERREUR("<UNK> n'est pas dans le lexique","");
      }

     if ((1)||(Recherche2Gram(m0,m1,&lp,&fr,Table_ML[RefML])))
      {
      nb_big++;

      retour=gram_proba_to_bigram(&pb_ref,m0,m1,RefML,0);
      if (retour==GRAM_CODE_UNKNOWN)
       {
       fprintf(stderr,"Warning : (RefML) GRAM_CODE_UNKNOWN sur %s : on met -99\n",bigram);
       pb_ref=-99;
       }
      else
       if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }

      if (verbose>4) printf("\tML(ref)=%lf\n",pb_ref);

      ref_perplex+=(-pb_ref);

      for(feuille=0;feuille<NbFeuilles;feuille++)
       {
       if ((1)||(Recherche2Gram(m0,m1,&lp,&fr,Table_ML[feuille])))
	{
	nbbigfeuille[feuille]++;
        retour=gram_proba_to_bigram(&pb,m0,m1,TablML[feuille],0);
        if (retour==GRAM_CODE_UNKNOWN)
 	 {
	 fprintf(stderr,"Warning : (ML feuille) GRAM_CODE_UNKNOWN sur %s : on met -99\n",bigram);
	 pb=-99;
	 }
        else
         if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }

        if (verbose>4) printf("\tML(%3d)=%lf\n",feuille,pb);

        tabl_perplex[feuille].pp+=(-pb);
        /*tabl_perplex[feuille].pp+=(-(TablCoeff[feuille].coeff_ref*pb_ref+TablCoeff[feuille].coeff_noeud*pb));*/
	}
       }
      } 
     }
    }
   pt=strtok(NULL," \t\n");
   }
  }
 
 ref_perplex=exp(ref_perplex/(double)nb_big*log(10.0));

 if (verbose>1) printf("TEST: %s - ppref=%.2lf\n",graf,ref_perplex);

 /* on calcule les perplexites */
 for(feuille=0;feuille<NbFeuilles;feuille++)
  if (nbbigfeuille[feuille]==0) tabl_perplex[feuille].pp=999999;
  else tabl_perplex[feuille].pp=exp(tabl_perplex[feuille].pp/(double)nbbigfeuille[feuille]*log(10.0));

 /* on trie les perplexites */
 qsort(tabl_perplex,NbFeuilles,sizeof(type_pp),compar_perplex);

 for(feuille=0;feuille<NbFeuilles;feuille++)
  {
  if (lia_recherche_avl(TablGrafTestAVL[tabl_perplex[feuille].feuille],graf,&i))
   {
   si_trouvable=1; place+=feuille;
   if (verbose>1)
    printf("\tplace=%d feuille[%d]=%d - present=%s - pp=%.2lf\n",feuille,tabl_perplex[feuille].feuille,
     TablFeuille[tabl_perplex[feuille].feuille],
     lia_recherche_avl(TablGrafTestAVL[tabl_perplex[feuille].feuille],graf,&i)?"OUI":"NON",
     tabl_perplex[feuille].pp<8000?tabl_perplex[feuille].pp:-1); 
   }
  }

/*  on ne considere pas la ref_perplex

 if (tabl_perplex[0].pp>ref_perplex)
  {
  nb_nspp++;
  if (verbose>1) printf("\t  Choix=ref_perplex\n");
  }
 else
*/

  {
  for (sibon=k=0;(k<MIN(n_premiers,NbFeuilles))&&(!sibon);k++)
   if (lia_recherche_avl(TablGrafTestAVL[tabl_perplex[k].feuille],graf,&i)) sibon=1;
  if (sibon)
   {
   nb_good++;
   if (verbose>1) printf("\t  Choix=%d -> GOOD (place=%d)\n",tabl_perplex[k].feuille,place);
   }
  else
   {
   nb_baad++;
   if (verbose>1) printf("\t  Choix=%d -> BAAD\n",tabl_perplex[0].feuille);
   }
  }
 /*
 if (verbose>1)
  {
  for (i=0;(i<NbFeuilles)&&(!lia_recherche_avl(TablGrafTestAVL[i],graf,&j));i++);
  printf("\tcorrect = %d\n",i);
  }
 */

 if (verbose>1)
  printf("\t\tNb de tests : %d : nb_good=%d (%.2lf%%) - nb_baad=%d (%.2lf%%) - nb_nspp=%d (%.2lf%%)\n",
   nb,nb_good,(double)(nb_good*100)/(double)nb,nb_baad,(double)(nb_baad*100)/(double)nb,
   nb_nspp,(double)(nb_nspp*100)/(double)nb);

 /*if (verbose>0) printf("%s\t%d\n",graf,tabl_perplex[0].pp==-1?0:TablFeuille[tabl_perplex[0].feuille]);*/
 /* maintenant on sort la liste de n_premiers premiers */
 
 printf("%s",sauvreference);
 for (k=0;k<MIN(n_premiers,NbFeuilles);k++) printf(" %d",TablFeuille[tabl_perplex[k].feuille]);
 printf("\n");
 

 if (si_trouvable) nb_trouvable++;
 }

nb--;

fprintf(stderr,"Nb de tests : %d : nb_good=%d (%.2lf%%) - nb_baad=%d (%.2lf%%) - nb_nspp=%d (%.2lf%%)\n",
 nb,nb_good,(double)(nb_good*100)/(double)nb,nb_baad,(double)(nb_baad*100)/(double)nb,
 nb_nspp,(double)(nb_nspp*100)/(double)nb);

fprintf(stderr,"Nb de mots trouvables : %d sur %d (%.2f%%)\n",nb_trouvable,nb,(float)(nb_trouvable*100)/(float)nb);
fprintf(stderr,"Place moyenne de la bonne solution (ds les trouvables) : %.2f\n",(float)place/(float)nb_trouvable);

free(nbbigfeuille);
free(tabl_perplex);
}

void determine_corpus_KL(char *fichtest)
{
int nb,i,j,feuille,nb_big,i_min,nb_good,nb_baad,nb_nspp,nb_mot;
char ch[TailleLigne*10],*pt,mot0[TailleLigne],mot1[TailleLigne],bigram[TailleLigne],graf[TailleLigne];
FILE *file;
wrd_index_t m0,m1;
err_t retour;
double *tabl_perplex,ref_perplex,pb,pb_ref,pp_min,kl_min;
float lp,fr;
type_kl *vecteur_kl;
ty_lexique lex_kl;

if (verbose) printf("on utilise des vecteurs kl de taille = %d\n",NbComposanteKL);
/* chargement du lexique correspondant aux kl */
sprintf(ch,"%s.niv%d",ARBRE,INDICE_KL);
lex_kl=ChargeLexique(ch);
vecteur_kl=(type_kl *)malloc(sizeof(type_kl)*NbComposanteKL);

tabl_perplex=(double*)malloc(sizeof(double)*NbFeuilles);

nb_good=nb_baad=nb_nspp=0;

if (!(file=fopen(fichtest,"rt"))) ERREUR("can't open :",fichtest);
for (nb=1;fgets(ch,TailleLigne*10,file);nb++)
 {
 /* initialisation du 'vecteur_kl' */
 for (i=0;i<NbComposanteKL;i++) { vecteur_kl[i].code=TablKL[0][i].code; vecteur_kl[i].pb=(float)0; }
 nb_mot=0;

 for(feuille=0;feuille<NbFeuilles;feuille++) tabl_perplex[feuille]=(double)0;
 ref_perplex=(double)0;
 nb_big=0;

 mot0[0]=mot1[0]='\0';
 pt=strtok(ch," \t\n");
 strcpy(graf,pt);
 pt=strtok(NULL," \t\n");
 if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb+1,fichtest); exit(0); }

 /*if (!strcmp(graf,"art_249_18")) verbose=2; else verbose=0;*/

 while (pt)
  {
  for(i=0;i<NB_NIVEAUX;i++)
   {
   if (pt==NULL) { fprintf(stderr,"ERREUR : ligne %d incorrect dans le fichier %s\n",nb+1,fichtest); exit(0); }

   if ((i+1)==INDICE_KL)
    {
    /* le mot est dans pt */
    if (!Mot2Code(pt,&m0,lex_kl)) fprintf(stderr,"Warning : '%s' inconnu dans 'lex_kl'\n",pt);
    else
     {
     vecteur_kl[acces_composante_kl(vecteur_kl,m0)].pb++;
     nb_mot++;
     }
    }

   if ((i+1)==INDICE_ML)
    {
    strcpy(mot0,mot1); strcpy(mot1,pt);
    if (mot0[0]!='\0')
     {
     /* on est sur un bigramme !! */
     sprintf(bigram,"%s|%s",mot0,mot1);

     if (verbose>2) printf("bigramme : %s\n",bigram);

     if (!Mot2Code(mot0,&m0,LexiqueGram))
      {
      /*ERREUR("Mot inconnu :",mot0);*/
      fprintf(stderr,"Warning : %s inconnu\n",mot0);

      if (!Mot2Code("<UNK>",&m0,LexiqueGram)) ERREUR("<UNK> n'est pas dans le lexique","");
      }
     if (!Mot2Code(mot1,&m1,LexiqueGram))
      {
      /*ERREUR("Mot inconnu :",mot1);*/
      fprintf(stderr,"Warning : %s inconnu\n",mot1);

      if (!Mot2Code("<UNK>",&m1,LexiqueGram)) ERREUR("<UNK> n'est pas dans le lexique","");
      }

     if ((1)||(Recherche2Gram(m0,m1,&lp,&fr,Table_ML[RefML])))
      {
      nb_big++;

      retour=gram_proba_to_bigram(&pb_ref,m0,m1,RefML,0);
      if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (RefML) GRAM_CODE_UNKNOWN sur :",bigram);
      if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }

      if (verbose>4) printf("\tML(ref)=%lf\n",pb_ref);

      ref_perplex+=(-pb_ref);

      for(feuille=0;feuille<NbFeuilles;feuille++)
       {
       retour=gram_proba_to_bigram(&pb,m0,m1,TablML[feuille],0);
       if (retour==GRAM_CODE_UNKNOWN) ERREUR("Pourquoi (ML feuille) GRAM_CODE_UNKNOWN sur :",bigram);
       if (retour!=CORRECT) { fprintf(stderr,"ERREUR : retour gram_proba_to_bigram = %d\n",retour); exit(0); }

       if (verbose>4) printf("\tML(%3d)=%lf\n",feuille,pb);

       tabl_perplex[feuille]+=(-pb);
       /*tabl_perplex[feuille]+=(-(TablCoeff[feuille].coeff_ref*pb_ref+TablCoeff[feuille].coeff_noeud*pb));*/
       }
      } 
     }
    }
   pt=strtok(NULL," \t\n");
   }
  }
 /* on calcule les probas sur le vecteur kl */
 if (nb_mot) for(i=0;i<NbComposanteKL;i++) vecteur_kl[i].pb/=(float)nb_mot;

 for(i_min=-1,feuille=0,kl_min=1000;feuille<NbFeuilles;feuille++)
  {
  lp=calcule_kl(vecteur_kl,TablKL[feuille]);
  if (lp<kl_min) { kl_min=lp; i_min=feuille; }
  if (verbose>1) printf("\tKL(%d) : dist=%f\n",feuille,lp);
  }
 
 ref_perplex=exp(ref_perplex/(double)nb_big*log(10.0));

 if (verbose>1) printf("TEST: %s - ppref=%.2lf\n",graf,ref_perplex);

 for(i_min=-1,feuille=0,pp_min=ref_perplex*kl_min;feuille<NbFeuilles;feuille++)
  {
  tabl_perplex[feuille]=exp(tabl_perplex[feuille]/(double)nb_big*log(10.0));
  tabl_perplex[feuille]*=calcule_kl(vecteur_kl,TablKL[feuille]);

  if (tabl_perplex[feuille]<pp_min) { i_min=feuille; pp_min=tabl_perplex[feuille]; }
 
  if (verbose>1) 
   printf("\tfeuille[%d]=%d - present=%s - pp=%.2lf\n",feuille,TablFeuille[feuille],
    lia_recherche_avl(TablGrafTestAVL[feuille],graf,&i)?"OUI":"NON",
    tabl_perplex[feuille]<8000?tabl_perplex[feuille]:-1); 
  }



 if (i_min==-1)
  {
  nb_nspp++;
  if (verbose>1) printf("\t  Choix=ref_perplex\n");
  }
 else
  if (lia_recherche_avl(TablGrafTestAVL[i_min],graf,&i))
   {
   nb_good++;
   if (verbose>1) printf("\t  Choix=%d -> GOOD\n",i_min);
   }
  else
   {
   nb_baad++;
   if (verbose>1) printf("\t  Choix=%d -> BAAD\n",i_min);
   }

 if (verbose>1)
  {
  for (i=0;(i<NbFeuilles)&&(!lia_recherche_avl(TablGrafTestAVL[i],graf,&j));i++);
  printf("\tcorrect = %d\n",i);
  }

 if (verbose>1)
  printf("\t\tNb de tests : %d : nb_good=%d (%.2lf%%) - nb_baad=%d (%.2lf%%) - nb_nspp=%d (%.2lf%%)\n",
   nb,nb_good,(double)(nb_good*100)/(double)nb,nb_baad,(double)(nb_baad*100)/(double)nb,
   nb_nspp,(double)(nb_nspp*100)/(double)nb);

 if (verbose>0) printf("%s\t%d\n",graf,i_min==-1?0:TablFeuille[i_min]);

 }

nb--;
printf("Nb de tests : %d : nb_good=%d (%.2lf%%) - nb_baad=%d (%.2lf%%) - nb_nspp=%d (%.2lf%%)\n",
 nb,nb_good,(double)(nb_good*100)/(double)nb,nb_baad,(double)(nb_baad*100)/(double)nb,
 nb_nspp,(double)(nb_nspp*100)/(double)nb);

free(vecteur_kl);
free(tabl_perplex);
}

/*................................................................*/

int main(int argc,char **argv)
{
char ch[TailleLigne],*pt;
int nb;
FILE *file;

if (argc<2)
 {
 fprintf(stderr,"Syntaxe : %s <fichier descripteur> [-compile]\n",argv[0]);
 exit(0);
 }

if (!(file=fopen(argv[1],"rt"))) ERREUR("can't open ",argv[1]);

/* init */

NB_NIVEAUX=INDICE_KL=INDICE_ML=TAILLE_FEUILLE=0;
CORPUS_TRAIN[0]=CORPUS_TEST[0]=CORPUS_DEV[0]=ARBRE[0]=
FEUILLE_REP[0]=TEST_REP[0]=NOEUD_REP[0]=SCRIPT_REP[0]='\0';

for(nb=0;fgets(ch,TailleLigne,file);nb++)
 {
 if (!strncmp(ch,"NB_NIVEAUX",10))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  sscanf(pt,"%d",&NB_NIVEAUX);
  } else
 if (!strncmp(ch,"INDICE_KL",9))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  sscanf(pt,"%d",&INDICE_KL);
  } else
 if (!strncmp(ch,"INDICE_ML",9))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  sscanf(pt,"%d",&INDICE_ML);
  } else
 if (!strncmp(ch,"TAILLE_FEUILLE",14))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  sscanf(pt,"%d",&TAILLE_FEUILLE);
  } else
 if (!strncmp(ch,"CORPUS_TRAIN",12))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(CORPUS_TRAIN,pt);
  } else
 if (!strncmp(ch,"CORPUS_TEST",11))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(CORPUS_TEST,pt);
  } else
 if (!strncmp(ch,"CORPUS_DEV",10))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(CORPUS_DEV,pt);
  } else
 if (!strncmp(ch,"ARBRE",5))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(ARBRE,pt);
  } else
 if (!strncmp(ch,"FEUILLE_REP",11))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(FEUILLE_REP,pt);
  } else
 if (!strncmp(ch,"TEST_REP",8))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(TEST_REP,pt);
  } else
 if (!strncmp(ch,"NOEUD_REP",9))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(NOEUD_REP,pt);
  } else
 if (!strncmp(ch,"SCRIPT_REP",10))
  {
  pt=strtok(ch," \t\n"); pt=strtok(NULL," \t\n");
  strcpy(SCRIPT_REP,pt);
  }
 }

fclose(file);

if (NB_NIVEAUX==0) ERREUR("NB_NIVEAUX non defini","");
if (INDICE_KL==0) ERREUR("INDICE_KL non defini","");
if (INDICE_ML==0) ERREUR("INDICE_ML non defini","");
if (TAILLE_FEUILLE==0) ERREUR("TAILLE_FEUILLE non defini","");

if (CORPUS_TRAIN[0]==0) ERREUR("CORPUS_TRAIN non defini","");
if (CORPUS_TEST[0]==0) ERREUR("CORPUS_TEST non defini","");
if (CORPUS_DEV[0]==0) ERREUR("CORPUS_DEV non defini","");
if (ARBRE[0]==0) ERREUR("ARBRE non defini","");
if (FEUILLE_REP[0]==0) ERREUR("FEUILLE_REP non defini","");
if (TEST_REP[0]==0) ERREUR("TEST_REP non defini","");
if (NOEUD_REP[0]==0) ERREUR("NOEUD_REP non defini","");
if (SCRIPT_REP[0]==0) ERREUR("SCRIPT_REP non defini","");

/* on lit les numeros de feuilles */
sprintf(ch,"%s.list_feuille",ARBRE);
lecture_liste_feuille(ch);

/* on charge le lexique */
sprintf(ch,"%s.niv%d",ARBRE,INDICE_ML);
LexiqueGram=ChargeLexique(ch);

/* on rempli les AVL */
rempli_arbre();

/* on compile les ML si demande */
if ((argc==3)&&(!strcmp(argv[2],"-compile"))) compile_arpa_noeud();

/* on calcule les vecteurs kl si demande */
if ((argc==3)&&(!strcmp(argv[2],"-kl")))
 {
 genere_kl();
 exit(0);
 }

/* on charge les ML */
charge_modele_arpa();

if ((argc==4)&&(!strcmp(argv[2],"-test")))
 {
 /*charge_kl();*/
 determine_corpus(argv[3]);
 }
else
/* on analyse la perplexite sur le test */
/*analyse_perplexite()*/;

/* on analyse la coherence des modeles */
/*coherence();*/

}
 
